#!/usr/bin/perl
#
# PGMR: (c) 2016, astx (at) aws-it.at
#
use strict;

our $soa_data=undef;
our $traces=undef;

if (-f "generate_soa_plot.soa_db") {
	require "generate_soa_plot.soa_db";
} else {
	die "soa database file not found";
}
if (-f "generate_soa_plot.input") {
	require "generate_soa_plot.input";
} else {
	die "your soa input/config file wasn't found";
}

# trace index
my $ti=0;
# output struct
my $o={};

foreach my $t (@{$traces}) {
	my $type=$t->{'soa_of'};
	print STDERR "generating SOA trace: $type\n";
	# read soa data:
	if (defined $soa_data->{"$type"}) {
		$ti++;
		print STDERR "trace $ti:\tfound SOA data for $type\n";
		my $s= $soa_data->{"$type"}{'soa'};
		my $d= $soa_data->{"$type"}{'desc'};

		my $xf=undef;
		my $xt=undef;
		my $yf=undef;
		my $yt=undef;
		my @l=();

		foreach my $e (@{$s}) {
			print STDERR "\t\tSOA line:\t";
			my ($vf, $af, $vt, $at)=split(/\s+/, $e);
			# get limits: lowest and highest x/y vals
			if (defined $xf) {
				if ($vf < $xf) { $xf=$vf; }
				if ($vt < $xf) { $xf=$vt; }

				if ($vf > $xt) { $xt=$vf; }
				if ($vt > $xt) { $xt=$vt; }

				if ($af < $yf) { $yf=$af; }
				if ($at < $yf) { $yf=$at; }

				if ($af > $yt) { $yt=$af; }
				if ($at > $yt) { $yt=$at; }
			} else {
				$xf=$vf;
				$xt=$vt;
				$yf=$af;
				$yt=$at;
			} 
			# skip zero length line:
			if ($vf == $vt && $af == $at) {
				print STDERR "\t\tskip zero length Line\n";
			} else {
				print STDERR "\t\tLine: \"A\" 4 2 ($vf,$af) ($vt,$at)\n";
				push(@l, "      Line: \"A\" 4 2 ($vf,$af) ($vt,$at)");
			}
		}


		#      Text: "A" 1 (50,6) ;TTC5200 OPS
		# TODO: better calculate position in graph
		my $tx=$xf*2;
		my $ty=$yt/2;
		if ($d ne '') {
			my $tyy=$ty/2;
			push(@l, "      Text: \"A\" 1 ($tx,$tyy) ;$d");
			push(@l, "      Text: \"A\" 1 ($tx,$ty) ;$t->{soa_of}: $t->{desc}");
		} else {
			push(@l, "      Text: \"A\" 1 ($tx,$ty) ;$t->{soa_of}: $t->{desc}");
		}
		# extends
		print STDERR "\t\tlowest  V: $xf\t\tA: $yf\n";
		print STDERR "\t\thighest V: $xt\t\tA: $yt\n";

		# add some extra headroom to display
		$xt+=$xt/20;
		$yt+=$yt/20;

		my @h=();
		push(@h, "      traces: 1 \{$ti,0,\"abs($t->{current_sensor})\"\}");
		push(@h, "      Parametric: \"abs($t->{diff_voltage})\"");
		push(@h, "      X: (' ',0,$xf,10,$xt)");
		push(@h, "      Y: ('_',1,$yf,0.99,$yt)");
		push(@h, "      Amps: ('_',1,0,1,$yf,0.99,$yt)");
		push(@h, "      Log: 1 1 0");
		push(@h, "      GridStyle: 1");

		$o->{"$ti"}{'H'}=\@h;
		$o->{"$ti"}{'L'}=\@l;
	}
}


print qq~[Transient Analysis]
{
   Npanes: $ti
   Active Pane: 1
~;

for my $i (1 .. $ti) {
	print "   {\n";

	print join("\n", @{$o->{"$i"}{'H'}}); print "\n";
	print join("\n", @{$o->{"$i"}{'L'}}); print "\n";

	print "   },\n";
}

print "}\n";

# EXAMPLE SOA DATABASE FILE: generate_soa_plot.soa_db:
# -----------------------------------------------------------------------------
#	#
#	# PGMR: (c) 2016, astx (at) aws-it.at
#	#
#	our $soa_data	=	{
#		'KSC3503'	=>	{
#			'soa'	=>	[
#				# extents/lines of SOA table
#				# from top left to bottom right
#				# V-from	A-from	V-to	A-to
#				'1	0.1	13	0.1',
#				'13	0.1	200	0.004',
#				'200	0.004	300	0.0024',
#				'1	0.1	70	0.1',
#				'70	0.1	300	0.012',
#				'300	0.012	300	0.001',
#			],
#			'desc'	=>	'DC operation uncooled/cooled',
#		},
#		'2SC4793'	=>	{
#			'soa'	=>	[
#				# extents/lines of SOA table
#				# from top left to bottom right
#				# V-from	A-from	V-to	A-to
#				'1	1	17	1',
#				'17	1	90	0.1',
#				'90	0.1	230	0.025',
#				'230	0.025	230	0.01',
#			],
#			'desc'	=>	'DC operation',
#		},
#		'2SA1837'	=>	{
#			'soa'	=>	[
#				# extents/lines of SOA table
#				# from top left to bottom right
#				# V-from	A-from	V-to	A-to
#				'1	1	17	1',
#				'17	1	90	0.1',
#				'90	0.1	230	0.025',
#				'230	0.025	230	0.01',
#			],
#			'desc'	=>	'DC operation',
#		},
#		'KSA1381'	=>	{
#			'soa'	=>	[
#				# extents/lines of SOA table
#				# from top left to bottom right
#				# V-from	A-from	V-to	A-to
#				'1	0.1	13	0.1',
#				'13	0.1	200	0.004',
#				'200	0.004	300	0.0024',
#				'1	0.1	70	0.1',
#				'70	0.1	300	0.012',
#				'300	0.012	300	0.001',
#			],
#			'desc'	=>	'DC operation uncooled/cooled',
#		},
#		'EXC10N20'	=>	{
#			'soa'	=>	[
#				# extents/lines of SOA table
#				# from top left to bottom right
#				# V-from	A-from	V-to	A-to
#				'1	10	1	10',	# use this zero length line to set upper limit for view
#				'1	8	15	8',
#				'15	8	160	0.8',
#				'160	0.8	160	0.1',
#			],
#			'desc'	=>	'DC operation single pulse',
#		},
#		'EXC10P20'	=>	{
#			'soa'	=>	[
#				# extents/lines of SOA table
#				# from top left to bottom right
#				# V-from	A-from	V-to	A-to
#				'1	10	1	10',	# use this zero length line to set upper limit for view
#				'1	8	15	8',
#				'15	8	160	0.8',
#				'160	0.8	160	0.1',
#			],
#			'desc'	=>	'DC operation single pulse',
#		},
#	};
#	
#	# to make perl happy you need this here:
#	1;
# -----------------------------------------------------------------------------

# EXAMPLE CONFIG/INPUT FILE: generate_soa_plot.input:
# -----------------------------------------------------------------------------
#	#
#	# PGMR: (c) 2016, astx (at) aws-it.at
#	#
#	
#	our $traces	=	[
#		{
#			'soa_of'		=>	'EXC10N20'		,
#			'current_sensor'	=>	'Is(M1)'		,
#			'diff_voltage'		=>	'V(+V,Vem)'		,
#			'desc'			=>	'OPS'			,
#		},
#		{
#			'soa_of'		=>	'EXC10P20'		,
#			'current_sensor'	=>	'Is(M2)'		,
#			'diff_voltage'		=>	'V(Vnem,-V)'		,
#			'desc'			=>	'OPS'			,
#		},
#		{
#			'soa_of'		=>	'KSA1381'		,
#			'current_sensor'	=>	'Ie(Q7)'		,
#			'diff_voltage'		=>	'V(VAS1,BIASH)'		,
#			'desc'			=>	'VAS'			,
#		},
#		{
#			'soa_of'		=>	'KSC3503'		,
#			'current_sensor'	=>	'Ie(Q10)'		,
#			'diff_voltage'		=>	'V(BIASL,Vccs2)'	,
#			'desc'			=>	'CCS'			,
#		},
#		{
#			'soa_of'		=>	'2SC4793'		,
#			'current_sensor'	=>	'Ie(Q11)'		,
#			'diff_voltage'		=>	'V(+V,Vemdrv)'		,
#			'desc'			=>	'OPS driver'		,
#		},
#		{
#			'soa_of'		=>	'2SA1837'		,
#			'current_sensor'	=>	'Ie(Q17)'		,
#			'diff_voltage'		=>	'V(-V,Vnemdrv)'		,
#			'desc'			=>	'OPS driver'		,
#		},
#	];
#	
#	# to make perl happy you need this here:
#	1;
# -----------------------------------------------------------------------------


# GENERATED ltspice plot file example:
# -----------------------------------------------------------------------------
# [Transient Analysis]
# {
#    Npanes: 6
#    Active Pane: 1
#    {
#       traces: 1 {1,0,"abs(Is(M1))"}
#       Parametric: "abs(V(+V,Vem))"
#       X: (' ',0,1,10,168)
#       Y: ('_',1,0.1,0.99,10.5)
#       Amps: ('_',1,0,1,0.1,0.99,10.5)
#       Log: 1 1 0
#       GridStyle: 1
#       Line: "A" 4 2 (1,8) (15,8)
#       Line: "A" 4 2 (15,8) (160,0.8)
#       Line: "A" 4 2 (160,0.8) (160,0.1)
#       Text: "A" 1 (2,2.5) ;DC operation single pulse
#       Text: "A" 1 (2,5) ;EXC10N20: OPS
#    },
#    {
#       traces: 1 {2,0,"abs(Is(M2))"}
#       Parametric: "abs(V(Vnem,-V))"
#       X: (' ',0,1,10,168)
#       Y: ('_',1,0.1,0.99,10.5)
#       Amps: ('_',1,0,1,0.1,0.99,10.5)
#       Log: 1 1 0
#       GridStyle: 1
#       Line: "A" 4 2 (1,8) (15,8)
#       Line: "A" 4 2 (15,8) (160,0.8)
#       Line: "A" 4 2 (160,0.8) (160,0.1)
#       Text: "A" 1 (2,2.5) ;DC operation single pulse
#       Text: "A" 1 (2,5) ;EXC10P20: OPS
#    },
#    {
#       traces: 1 {3,0,"abs(Ie(Q7))"}
#       Parametric: "abs(V(VAS1,BIASH))"
#       X: (' ',0,1,10,315)
#       Y: ('_',1,0.001,0.99,0.105)
#       Amps: ('_',1,0,1,0.001,0.99,0.105)
#       Log: 1 1 0
#       GridStyle: 1
#       Line: "A" 4 2 (1,0.1) (13,0.1)
#       Line: "A" 4 2 (13,0.1) (200,0.004)
#       Line: "A" 4 2 (200,0.004) (300,0.0024)
#       Line: "A" 4 2 (1,0.1) (70,0.1)
#       Line: "A" 4 2 (70,0.1) (300,0.012)
#       Line: "A" 4 2 (300,0.012) (300,0.001)
#       Text: "A" 1 (2,0.025) ;DC operation uncooled/cooled
#       Text: "A" 1 (2,0.05) ;KSA1381: VAS
#    },
#    {
#       traces: 1 {4,0,"abs(Ie(Q10))"}
#       Parametric: "abs(V(BIASL,Vccs2))"
#       X: (' ',0,1,10,315)
#       Y: ('_',1,0.001,0.99,0.105)
#       Amps: ('_',1,0,1,0.001,0.99,0.105)
#       Log: 1 1 0
#       GridStyle: 1
#       Line: "A" 4 2 (1,0.1) (13,0.1)
#       Line: "A" 4 2 (13,0.1) (200,0.004)
#       Line: "A" 4 2 (200,0.004) (300,0.0024)
#       Line: "A" 4 2 (1,0.1) (70,0.1)
#       Line: "A" 4 2 (70,0.1) (300,0.012)
#       Line: "A" 4 2 (300,0.012) (300,0.001)
#       Text: "A" 1 (2,0.025) ;DC operation uncooled/cooled
#       Text: "A" 1 (2,0.05) ;KSC3503: CCS
#    },
#    {
#       traces: 1 {5,0,"abs(Ie(Q11))"}
#       Parametric: "abs(V(+V,Vemdrv))"
#       X: (' ',0,1,10,241.5)
#       Y: ('_',1,0.01,0.99,1.05)
#       Amps: ('_',1,0,1,0.01,0.99,1.05)
#       Log: 1 1 0
#       GridStyle: 1
#       Line: "A" 4 2 (1,1) (17,1)
#       Line: "A" 4 2 (17,1) (90,0.1)
#       Line: "A" 4 2 (90,0.1) (230,0.025)
#       Line: "A" 4 2 (230,0.025) (230,0.01)
#       Text: "A" 1 (2,0.25) ;DC operation
#       Text: "A" 1 (2,0.5) ;2SC4793: OPS driver
#    },
#    {
#       traces: 1 {6,0,"abs(Ie(Q17))"}
#       Parametric: "abs(V(-V,Vnemdrv))"
#       X: (' ',0,1,10,241.5)
#       Y: ('_',1,0.01,0.99,1.05)
#       Amps: ('_',1,0,1,0.01,0.99,1.05)
#       Log: 1 1 0
#       GridStyle: 1
#       Line: "A" 4 2 (1,1) (17,1)
#       Line: "A" 4 2 (17,1) (90,0.1)
#       Line: "A" 4 2 (90,0.1) (230,0.025)
#       Line: "A" 4 2 (230,0.025) (230,0.01)
#       Text: "A" 1 (2,0.25) ;DC operation
#       Text: "A" 1 (2,0.5) ;2SA1837: OPS driver
#    },
# }
# -----------------------------------------------------------------------------
